/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.j;

import java.util.*;

public class SetUtils {

	/**
	 * Fill set from string by using the given separator.
	 * @param set
	 * @param source
	 * @param sep source is split by this string
	 * @param trim if true strings are trimmed before being added to set
	 * @return
	 */
	public static Set populateSetFromString(Set set, String source, String sep, boolean trim)
	{
		String[] a = source.split(sep);
		for(int i = 0; i<a.length; i++) {
			if(trim)
				set.add(a[i].trim());
			else
				set.add(a[i]);
		}
		return set;
	}
	
	/**
	 * Fills a given Set with values from the given array
	 * @param set
	 * @param array
	 * @param offset - start copy process from here
	 * @param step - and step through the array with this increment size
	 * @return
	 */
	public static Set fill(Set set, Object[] array, int offset, int step) 
	{
		for (int i=offset; i<array.length; i+=step) {
			set.add(array[i]);
		}
		return set;
	}
	
	//
	public static String concat(Set list, String separator, String lastSeparator) 
	{
		if(list == null)
			return null;
		StringBuffer b = new StringBuffer();
		if(separator==null)
			separator = "";
		if(lastSeparator==null)
			lastSeparator = separator;
		int counter = 0, last = list.size()-1;
		Iterator i = list.iterator();
		while(i.hasNext()) {
			if(counter > 0) {
				if(counter == last)
					b.append(lastSeparator);
				else
					b.append(separator);
			}
			b.append(i.next().toString());
			counter++;
		}
		return b.toString();
	}
}
