/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.jam;

import org.w3c.dom.Node;

public class JspAppTransition implements JspAppNode {

	//
	// Instance
	//
	
	private JspAppState parentState;
	private String condition;
	private String nextState;
	private String type;
	
	// 
	// Constructors
	//

	public JspAppTransition(JspAppState parent) {
		super();
		this.parentState = parent;
	}
	
	public JspAppState state() {
		return parentState;
	}
	
	//
	// Accessors
	//
	
	public String getCondition() {
		if(condition==null)
			return JAM_DEFAULT_CONDITION;
		else
			return condition;
	}

	public void setCondition(String str) {
		this.condition = str;
	}
	
	public String getNextState() {
		return nextState;
	}

	public void setNextState(String str) {
		this.nextState = str;
	}

	public String getType() {
		if(type==null)
			return JAM_GOTO_TYPE;
		else
			return type;
	}

	public void setType(String str) {
		this.type = str;
	}
	
	//
	// Stuff
	//
	
	public String toXML(String indent) {
		StringBuffer b = new StringBuffer();
		b.append("\n"+indent+"<"+getType()+">");
		b.append("\n"+indent+PP_INDENT+"<"+JAM_CONDITION_TAG+">" + getCondition() + "</"+JAM_CONDITION_TAG+">");
		b.append("\n"+indent+PP_INDENT+"<"+JAM_NEXT_TAG+">" + getNextState() + "</"+JAM_NEXT_TAG+">");
		b.append("\n"+indent+"</"+getType()+">");
		return b.toString();
	}

	//
	// Parsing
	//
	
	public JspAppNode parseNode(Node root)
	{
		Node child = root.getFirstChild();
		setType(root.getNodeName());
		//String initStateName = null;
		while( child != null) {
			short nt = child.getNodeType();
			switch(nt) {
				case Node.ELEMENT_NODE:
					String nn = child.getNodeName();
					if (nn.equals(JAM_CONDITION_TAG)) {
						setCondition(JspApp.collectText(child));
					}
					else if (nn.equals(JAM_NEXT_TAG)) {
						setNextState(JspApp.collectText(child));
					}
					break;
				default:		
					break;
			}
			child = child.getNextSibling();
		}
		return this;
	}

}
