/*
 * Copyright (c) 2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision$
 * $LastChangedDate$
 * 
 */

package com.codedroids.oc.forms;

import javax.servlet.jsp.PageContext;
import org.opencms.jsp.*;
import com.codedroids.forms.*;
import net.sf.json.JSONObject;

public interface I_PersistFormToRecord {
	
	/**
	 * Implementors may choose to do appropriate setup in this method which is called before the first invocation of
	 * <code>persist()</code>.
	 * @param config
	 * @param cmsjsp
	 * @param pageContext
	 * @throws PersistFormToRecordException
	 */
	public void init(JSONObject config, CmsJspActionElement cmsjsp, PageContext pageContext) throws PersistFormToRecordException;
	
	/**
	 * Implementors may choose remove resource from setup in this method which is called after the last invocation of
	 * <code>persist()</code>.
	 * @param config
	 * @param cmsjsp
	 * @param pageContext
	 */
	public void finish(JSONObject config,CmsJspActionElement cmsjsp, PageContext pageContext) throws PersistFormToRecordException;
	
	/**
	 * Called per form instance that need to be persisted
	 * @param form
	 * @param config
	 * @param cmsjsp
	 * @param pageContext
	 * @throws PersistFormToRecordException
	 */
	public void persist(Form form, JSONObject config, CmsJspActionElement cmsjsp, PageContext pageContext) throws PersistFormToRecordException;
}
