/*
 * Copyright (c) 2005-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 177 $
 * $LastChangedDate: 2007-05-07 11:38:46 +0200 (Mon, 07 May 2007) $
 * 
 */

package com.codedroids.oc.records;

import java.util.*;
import org.w3c.dom.*;
import com.codedroids.records.RecordFactory;
import com.codedroids.records.I_Record;
import com.codedroids.records.RecordException;
import com.codedroids.oc.OCUtils;
import com.codedroids.oc.PathUtils;
import com.codedroids.j.StringUtils;
//import com.opencms.boot.I_CmsLogChannels;
//import org.opencms.main.OpenCms;
import org.opencms.file.CmsObject;
import org.opencms.file.CmsResource;

public class OCRecordFactory extends RecordFactory {
	
	/**
	 * Loads xml-entity definition if it either is not registered already or the xml file
	 * has been modified after the instance was registered.
	 * 
	 * @param name - name of the entity - this is already in the xmlfile, but if caching is
	 *               to work it is needed so we do not have to load the file to get at it.
	 *               For convenience it will be deducted from the filename if name is null:
	 *               e.g. /foo/bar/Contact.rec.xml -> name = Contact
	 * @param cmso
	 * @param xmlfile
	 * @param reload
	 * @return
	 */
	private static HashMap fcache = new HashMap(); // caching per file, rather than entity
	public synchronized static RecordFactory registerFactory(CmsObject cmso, String xmlfile) throws RecordException
	{
		return registerFactory(null, cmso, xmlfile);
	}
	
	public synchronized static RecordFactory registerFactory(String name, CmsObject cmso, String xmlfile) throws RecordException
	{
		if(name==null)
			name = PathUtils.removeTrailing(PathUtils.filenameFromPath(xmlfile), FILE_EXT);
		RecordFactory factory = null;
		try {
			// Caches the document tree since it is the mod.date of xmlfile that determines if refresh is needed 
			Date timestamp = (Date)fcache.get(xmlfile);
			if(timestamp != null) {
				CmsResource f = cmso.readResource(xmlfile);
				if(f != null && f.getDateLastModified() <= timestamp.getTime())
					factory = getFactory(name); // it is safe to use cached factory if it exists
			}
			if(factory == null) {
				Element root = OCUtils.loadXMLFileFromVFS(cmso, xmlfile).getDocumentElement();
				factory = new OCRecordFactory();
				parseEntityFactory(factory, root);
				setFactory(factory.getName(), factory);
				fcache.put(xmlfile, new Date());
			}
		} catch (Exception e) {
			throw new RecordException(e);
		}
		return factory;
	}
	
	//
	// Subclasses should override these (should be made abstract at some point...)
	//
	public String c_pool_name = org.opencms.db.CmsDbPool.DBCP_JDBC_URL_PREFIX + "opencms:default"; 
	public String getPoolName() {
		return c_pool_name;
	}
	
	/**
	 * @deprecated
	 */
	public Integer getNextId(String tableName) {
		try {
			return new Integer(org.opencms.db.CmsDbUtil.nextId(getPoolName(), getTableName()));
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public Object getNextId(I_Record record) {
		try {
			if(getIdentScheme().equals(REC_ID_SCHEME_SEQ))
				return new Integer(org.opencms.db.CmsDbUtil.nextId(getPoolName(), getTableName()));
			else if(getIdentScheme().equals(REC_ID_SCHEME_UUID))
				return (new org.opencms.util.CmsUUID()).toString();
			else if(getIdentScheme().equals(REC_ID_SCHEME_RAND))
				return StringUtils.randomString(18);
			else if(getIdentScheme().equals(REC_ID_SCHEME_NONE)) {
				return record.valueForField(getIdentField());
			} else
				throw new Exception("Unknown ident scheme: \"" + getIdentScheme() + "\"");
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	
	public void debug(String msg) {
		System.out.println(msg);
//		A_OpenCms.log(I_CmsLogChannels.C_OPENCMS_INFO, "[FETCH RECORDS] = " + statement);
	}
}
