/*
 * Copyright (c) 2006-2007 CodeDroids ApS (http://www.codedroids.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * For further information about CodeDroids ApS, please see the
 * company website: http://www.codedroids.com
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $LastChangedRevision: 117 $
 * $LastChangedDate: 2007-02-20 11:24:48 +0100 (Tue, 20 Feb 2007) $
 * 
 */

package com.codedroids.templating;

import java.util.Map;
import java.util.HashMap;


public class TemplateEngineFactory {
	
	private static Map engineClassNames = new HashMap();
	/**
	 * All template engines must register with a factory before the factory can use them.
	 * The same class can be registered more than once if it makes sense to have it under more than one shortName
	 * (actually the className will always be added under the class name itself, which should provide for a unique
	 * entry under all circumstances)
	 * ShortName is a string with no particular meaning, as long as it is unique, file-extension could
	 * be a candidate for this (if a short version is needed). 
	 * 
	 * @param className name of class that must be instantiated for this engine
	 * @param shortName optional "short hand" ident.
	 */
	public static void registerTemplateEngineClassName(String className, String shortName) {
		engineClassNames.put(className, className);
		if(shortName != null)
			engineClassNames.put(shortName, className);
	}
	
	public static String templateEngineClassName(String key) {
		return (String)engineClassNames.get(key);
	}
	
	/**
	 * Currently a no-op - use the ditto in com.codedroids.oc.templating. 
	 * Eventuelly this should work with plain file-based templates
	 * 
	 * <p>Purpose is, based on parameters to load and instantiate a template engine, which subsequently can be invoked
	 * via the A_TemplateEngine interface.
	 * 
	 * @return
	 */
	public static A_TemplateEngine templateEngine() 
	{
		A_TemplateEngine instance = null;
		return instance;
	}
	


}
