<%--
  Copyright (c) 2004-2007 CodeDroids ApS (http://www.codedroids.com)
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  Lesser General Public License for more details.
  
  For further information about CodeDroids ApS, please see the
  company website: http://www.codedroids.com
  
  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
  $LastChangedRevision: 178 $
  $LastChangedDate: 2007-05-09 16:36:20 +0200 (Wed, 09 May 2007) $
--%><%--
  Generic form handler, saves the contents according to the given entity (if no entity is given this really
  is a no-op script)

  Form:            XML file in VFS that defines the form.
  FormIdent:       If given overrides the one from the definition, if it is not defined there either an exception is thrown
  ForwardPage:     Where to go when finished succesfully. If FinishTemplate/FinishFormat/FinishForm is set 
                   then this automated forwarding is suppressed otherwise their output would not be shown 
  FinishForm:      Values: "hidden" (default) or "visible". Determines if the form is shown once finished.
                   Note that using default rather setting "hidden" explicitly differ as the default won't
                   suppress forwarding as setting explicitly to hidden would.  
  FinishTemplate:  Filename for the template. If set FinishFormat determines various settings. If only the filename is
                   given it is looked up relative to the request path. If only a path is given a file is looked up
                   in that where the filename is based in the name of the request path/page - extension + type extension
  FinishFormat:    If FinishTemplate is set this provides further settings for that, list on the form: key:value;...
                   At least the type must be given, eg. "type:ftl" or "type:xsl", other settings depends on type. 
                   Otherwise, for backward compat. it serves as a simple template using ${...}, \n and \t are handled.
  FormValues:      Option for setting default values in fields, form: <field-name>:<field-value>[,<field-value>]*;
                   Files properties can be accessed as ${...}
  Entity:          If given a new record of the entity is created and saved with the values from the submitted form.
                   As a convenience, if the entity defines fields like "creator", "creationDate" and "lastIp" these will be populated
  Persistence:     Provides a more flexible approach than using the Entity-variation. Spec is on JSON format. It must define
                   either the "class" or "jsp" entries, e.g. {"jsp":"/foo/bar/save.jsp"}.
                   If "class" is given then the class is instantiated (must implement I_PersistFormToRecord interface).
                   In this case the instantiated class is responsible for interpreting the rest of the config info.
                   If "jsp" is given (but no "class") then the com.codedroids.oc.forms.PersistViaJsp is instantiated.
                   This class imports the JSP file into the current request context (after setting two page context attributes:
                   the com.codedroids.oc.forms.PersistViaJsp.form and the com.codedroids.oc.forms.PersistWithJsp.config. 
--%><%@ 
page session="false" %><%@ 
page import="java.util.*, 
             org.opencms.jsp.*,
             com.codedroids.j.*, com.codedroids.oc.forms.*,
             com.codedroids.templating.*, com.codedroids.oc.templating.*" 
%><%
  CmsJspActionElement cms = new CmsJspActionElement(pageContext, request, response);

  //out.println("<!--\n" + com.codedroids.j.MapUtils.prettyPrint(request.getParameterMap()) + "\n-->\n");
  // Params - none so far

  // Props
  String forwardPage = cms.property("FinishPage", null, null ); // cmso.getRequestContext().getUri()
  if (forwardPage != null)
    forwardPage = cms.link(forwardPage);
  String finishTemplate = cms.property("FinishTemplate", null, null);
  String finishFormat = cms.property("FinishFormat", null, null);
  if(finishFormat == null && finishTemplate!=null) {
    throw new Exception("The \"FinishFormat\" property must be set when the \"FinishTemplate\" property is set");
  }

  String finishForm = cms.property("FinishForm", null, null);

  String entityFile = cms.property("Entity");
  String persistConfig = cms.property("Persistence");
  
  String formFile = cms.property("Form", null, null);
  if(formFile == null || formFile.length() == 0)
    throw new Exception("The \"Form\" property must be set");

  OCForm form = OCForm.getInstance(cms, formFile);

  String formIdent = cms.property("FormIdent", null, null);
  if(formIdent != null)
    form.setIdent(formIdent);
  else if(form.getIdent() != null)
    formIdent = form.getIdent();
  else // formIdent == null && form.getIdent() == null)
    throw new Exception("The \"FormIdent\" property must be set");
  
  String formValuesStr = cms.property("FormValues", null, null);
  Map formValues = null;
  if(formValuesStr != null && formValuesStr.length() > 0)
	formValues = OCFormUtils.mapFromPropertyExpandedFormatString(cms, formValuesStr);

  //
  // Finalize setup of form
  //
  
  form.getLayout().setFormAction(cms.link(cms.getRequestContext().getUri()));
  form.setLocale(cms.getRequestContext().getLocale().toString());

  //
  // Start processing
  //

  boolean done = false;
  StringBuffer errors = new StringBuffer();

  if(form.isNotSubmitted(request)) {
    // first time, set fields to default if any
    if(formValues != null)
      form.setFieldValuesFromMap(formValues, null, true, true);
  } else {
    form.collectValues(request);
    int errorCnt = form.validateValues();
    if(errorCnt > 0) {
      errors.append("<div class=\"ErrorMessages\"><ol>");
      Iterator i = form.collectErrorMessages().iterator();
      while(i.hasNext()) {
        errors.append("<li>"+i.next());
      }
      errors.append("</ol></div>");
    } else {
      // Save if needed (if both entityFile and persistConfig is given both will be performed)
      try {
          if(persistConfig != null) {
            OCFormUtils.persistFormToRecord(form, persistConfig, cms);
     	    }
          if(entityFile != null) {
            OCFormUtils.saveFormToRecord(form, entityFile, cms);
          }
          done = true;
      } catch (Exception e) {
          errors.append("<div class=\"ErrorMessages\"><p>Exception=");
          errors.append(((e.getMessage()==null) ? e.toString() : e.getMessage()));
          errors.append("</p></div>");
          e.printStackTrace();
          done = false;
      }
    }
  }

  if(!done) {
    if(errors.length() > 0) {
      out.println(errors.toString());
    }
    out.println(form.toHTML());
  } else {
    if(finishFormat != null) {
      if(finishTemplate==null) {
        // old-style template then
        Map params = form.getFieldValues(null, ", ", " & ");
        finishFormat = finishFormat.replaceAll("\\\\n", "\n");
        finishFormat = finishFormat.replaceAll("\\\\t", "\t");
        out.println(StringUtils.replaceStrings(finishFormat, params, "${", "}"));
      } else {
          // flexible templating
          A_TemplateEngine te = OCTemplateEngineFactory.templateEngine(cms, finishTemplate, finishFormat);
          te.process(Collections.singletonMap("form", form.getValues()), out);
      }
    }
    if(finishForm != null && !finishForm.equals("hidden")) {
      out.println(form.toHTML());
    } 
    if(forwardPage != null && finishForm == null && finishTemplate == null && finishFormat == null) {
      response.sendRedirect(forwardPage);
    }
  }
%>